#include "Module.h"
#include "IRBuilder.h"
#include "Type.h"
#include "Constant.h"

#include <iostream>
#include <vector>

int main()
{
    auto mod        = new Module("assign.c");
    auto bud        = new IRBuilder(nullptr, mod);
    auto I32Type    = Type::get_int32_type(mod);
    auto I32x10Type = Type::get_array_type(I32Type, 10);

    std::vector<Type *> NoneArg;
    auto                mainTp   = FunctionType::get(I32Type, NoneArg);
    auto                mainFunc = Function::create(mainTp, "main", mod);

    auto bb                      = BasicBlock::create(mod, "entry", mainFunc);
    bud->set_insert_point(bb);

    auto arr_ptr = bud->create_alloca(I32x10Type);
    auto a0_ptr  = bud->create_gep(
         arr_ptr, {ConstantInt::get(0, mod), ConstantInt::get(0, mod)});
    auto a1_ptr = bud->create_gep(
        arr_ptr, {ConstantInt::get(0, mod), ConstantInt::get(0, mod)});
    bud->create_store(ConstantInt::get(10, mod), a0_ptr);
    auto a0_load = bud->create_load(a0_ptr);
    auto mul     = bud->create_imul(a0_load, ConstantInt::get(2, mod));
    bud->create_store(mul, a1_ptr);
    auto a1_load = bud->create_load(a1_ptr);
    bud->create_ret(a1_load);
    
    std::cout << mod->print();
    delete mod;
    return 0;
}
