#include "BasicBlock.h"
#include "Constant.h"
#include "Function.h"
#include "IRBuilder.h"
#include "Module.h"
#include "Type.h"

#include <iostream>

int main()
{
    auto mod = new Module("if.c");
    auto bud = new IRBuilder(nullptr, mod);
    Type *I32Type = Type::get_int32_type(mod);
    Type *floatType = Type::get_float_type(mod);

    auto mainTp = FunctionType::get(I32Type, {});
    auto mainFunc = Function::create(mainTp, "main", mod);
    auto bb = BasicBlock::create(mod, "entry", mainFunc);

    bud->set_insert_point(bb);
    auto aAlloc = bud->create_alloca(floatType);
    bud->create_store(ConstantFP::get(5.555, mod), aAlloc);
    auto aValue = bud->create_load(aAlloc);
    auto cond = bud->create_fcmp_gt(aValue, ConstantFP::get(1, mod));
    auto TBB = BasicBlock::create(mod, "trueCase", mainFunc);
    auto FBB = BasicBlock::create(mod, "falseCase", mainFunc);
    bud->create_cond_br(cond, TBB, FBB);

    bud->set_insert_point(TBB);
    bud->create_ret(ConstantInt::get(233, mod));
    bud->set_insert_point(FBB);
    bud->create_ret(ConstantInt::get(0, mod));

    std::cout << mod->print();
    delete mod;
    return 0;
}
