#include "BasicBlock.h"
#include "Constant.h"
#include "Function.h"
#include "IRBuilder.h"
#include "Module.h"
#include "Type.h"

#include <iostream>

int main()
{
    auto  mod = new Module("while.c");
    auto  bud     = new IRBuilder(nullptr, mod);
    Type *I32Type   = Type::get_int32_type(mod);

    // main函数
    auto mainFunc =
        Function::create(FunctionType::get(I32Type, {}), "main", mod);
    auto bb = BasicBlock::create(mod, "entry", mainFunc);
    auto judge = BasicBlock::create(mod, "judge", mainFunc);
    auto loop = BasicBlock::create(mod, "loop", mainFunc);
    auto exit = BasicBlock::create(mod, "exit", mainFunc);
    
    bud->set_insert_point(bb);
    auto aAlloc = bud->create_alloca(I32Type);
    auto iAlloc = bud->create_alloca(I32Type);
    bud->create_store(ConstantInt::get(10, mod), aAlloc);
    bud->create_store(ConstantInt::get(0, mod), iAlloc);
    bud->create_br(judge);
    
    bud->set_insert_point(judge);
    auto iV = bud->create_load(iAlloc);
    auto cond = bud->create_icmp_lt(iV, ConstantInt::get(10,  mod));
    bud->create_cond_br(cond, loop, exit);


    bud->set_insert_point(loop);
    auto add = bud->create_iadd(iV, ConstantInt::get(1, mod));
    bud->create_store(add, iAlloc);
    //
    auto aV = bud->create_load(aAlloc);
    auto aadd = bud->create_iadd(add, aV);
    bud->create_store(aadd, aAlloc);
    bud->create_br(judge);

    bud->set_insert_point(exit);
    aV = bud->create_load(aAlloc);
    bud->create_ret(aV);

    std::cout << mod->print();
    delete mod;
    return 0;
}
